@@ -1,5 +1,6 @@
 from dexy.handler import DexyHandler
-import pexpect
+import os
+import subprocess
 
 class AsciidocHandler(DexyHandler):
     """IN DEVELOPMENT. Converts .txt files with asciidoc markup to HTML or
@@ -9,12 +10,13 @@ class AsciidocHandler(DexyHandler):
     INPUT_EXTENSIONS = [".txt"]
     OUTPUT_EXTENSIONS = [".html", ".xml"]
     ALIASES = ['asciidoc']
+    FINAL = True
 
     def process(self):
         self.artifact.generate_workfile()
-        workfile = self.artifact.work_filename(False)
-        outfile = self.artifact.filename(False)
-        
+        workfile = self.artifact.work_filename()
+        outfile = self.artifact.filename()
+
         extension = self.artifact.ext
 
         if extension == ".html":
@@ -23,13 +25,25 @@ class AsciidocHandler(DexyHandler):
             backend = "docbook"
         else:
             raise Exception("unexpected file extension in asciidoc handler %s" % extension)
-        
+
         command = "%s %s -o %s %s" % (self.EXECUTABLE, backend, outfile, workfile)
         self.log.debug(command)
-        output = pexpect.run(command, cwd=self.artifact.artifacts_dir)
-        self.artifact.stdout = output
-        self.log.debug("\n%s" % output)
-        
-        f = open(self.artifact.filename(), "r")
+
+        if self.doc.args.has_key('env'):
+            env = os.environ
+            env.update(self.doc.args['env'])
+        else:
+            env = None
+
+        proc = subprocess.Popen(command, shell=True,
+                                cwd=self.artifact.artifacts_dir,
+                                stdout=subprocess.PIPE,
+                                stderr=subprocess.STDOUT,
+                                env=env)
+
+        stdout, stderr = proc.communicate()
+        self.artifact.stdout = stdout
+
+        f = open(self.artifact.filepath(), "r")
         self.artifact.data_dict['1'] = f.read()
         f.close()
