@@ -1,8 +1,4 @@
-try:
-    from collections import OrderedDict
-except ImportError:
-    from ordereddict import OrderedDict
-
+from ordereddict import OrderedDict
 from dexy.handler import DexyHandler
 from pynliner import Pynliner
 
@@ -14,12 +10,10 @@ class PynlinerHandler(DexyHandler):
     ALIASES = ['pynliner', 'inlinecss']
 
     def process_dict(self, input_dict):
-        #self.artifact.load_input_artifacts()
-        #print self.artifact.input_artifacts_dict.keys()
         #matches = [k for k in self.artifact.input_artifacts_dict.keys() if k.endswith(".css|dexy")]
         #k = matches[0]
         css = open("pastie.css", "r").read()
-        
+
         output_dict = OrderedDict()
         for k, v in input_dict.items():
             p = Pynliner()
