@@ -1,14 +1,7 @@
-try:
-    from collections import OrderedDict
-except ImportError:
-    from ordereddict import OrderedDict
-
 from dexy.handler import DexyHandler
-
 import os
 import subprocess
 
-### @export "stdout"
 class ProcessStdoutHandler(DexyHandler):
     """
     Runs python script and returns STDOUT.
@@ -22,12 +15,11 @@ class ProcessStdoutHandler(DexyHandler):
 
     def process(self):
         self.artifact.generate_workfile()
-        command = "%s %s" % (self.executable(), self.artifact.work_filename(False))
+        command = "%s %s" % (self.executable(), self.artifact.work_filename())
         cla = self.artifact.command_line_args()
-
-        if self.artifact.doc.args.has_key('env'):
+        if self.doc.args.has_key('env'):
             env = os.environ
-            env.update(self.artifact.doc.args['env'])
+            env.update(self.doc.args['env'])
         else:
             env = None
 
@@ -35,21 +27,25 @@ class ProcessStdoutHandler(DexyHandler):
             command = "%s %s" % (command, cla)
         self.log.debug(command)
 
-
         proc = subprocess.Popen(command, shell=True,
                                 cwd=self.artifact.artifacts_dir,
                                 stdout=subprocess.PIPE,
                                 stderr=subprocess.PIPE,
                                 env=env)
-        proc.wait()
-        self.artifact.data_dict['1'] = proc.stdout.read()
-        if proc.returncode != 0:
-            if self.artifact.doc.args.has_key('raiseerrors'):
-                raise "return code not 0!"
+        stdout, stderr = proc.communicate()
+        self.artifact.data_dict['1'] = stdout
+        self.artifact.stdout = stderr
+
+        if proc.returncode is None:
+            raise Exception("no return code, proc not finished!")
+        elif proc.returncode != 0:
+            if self.doc.controller.args.ignore_errors:
+                self.log.warn(stderr)
             else:
-                self.log.warn("an error occurred:\n%s" % proc.stderr)
+                print stderr
+                raise Exception("""proc returned nonzero status code! if you don't
+want dexy to raise errors on failed scripts then pass the --ignore-errors option""")
 
-### @export "bash"
 class BashHandler(ProcessStdoutHandler):
     """
     Runs bash script and returns STDOUT.
@@ -60,7 +56,6 @@ class BashHandler(ProcessStdoutHandler):
     OUTPUT_EXTENSIONS = [".txt"]
     ALIASES = ['bash']
 
-### @export "php"
 class PhpHandler(ProcessStdoutHandler):
     """
     Runs php script and returns STDOUT.
@@ -71,7 +66,6 @@ class PhpHandler(ProcessStdoutHandler):
     OUTPUT_EXTENSIONS = [".html", ".txt"]
     ALIASES = ['php']
 
-### @export "escript"
 class EscriptHandler(ProcessStdoutHandler):
     """
     Runs escript (erlang) and returns STDOUT.
@@ -81,8 +75,7 @@ class EscriptHandler(ProcessStdoutHandler):
     OUTPUT_EXTENSIONS = [".txt"]
     ALIASES = ['escript']
 
-### @export "luaout"
-class LuaStdoutHandler(ProcessStdoutHandler):
+class LuaHandler(ProcessStdoutHandler):
     """
     Runs lua script and returns STDOUT.
     """
@@ -90,9 +83,8 @@ class LuaStdoutHandler(ProcessStdoutHandler):
     VERSION = '/usr/bin/env lua -v'
     INPUT_EXTENSIONS = ['.lua']
     OUTPUT_EXTENSIONS = ['.txt']
-    ALIASES = ['luaout']
+    ALIASES = ['lua']
 
-### @export "redcloth"
 class RedclothHandler(ProcessStdoutHandler):
     """
     Runs redcloth, converts textile markup to HTML.
@@ -103,7 +95,6 @@ class RedclothHandler(ProcessStdoutHandler):
     OUTPUT_EXTENSIONS = [".html"]
     ALIASES = ['redcloth', 'textile']
 
-### @export "redclothl"
 class RedclothLatexHandler(ProcessStdoutHandler):
     """
     Runs redcloth, converts textile markup to LaTeX.
@@ -114,28 +105,24 @@ class RedclothLatexHandler(ProcessStdoutHandler):
     OUTPUT_EXTENSIONS = [".tex"]
     ALIASES = ['redclothl', 'latextile']
 
-### @export "rst2html"
 class Rst2HtmlHandler(ProcessStdoutHandler):
     EXECUTABLE = '/usr/bin/env rst2html.py'
     INPUT_EXTENSIONS = [".rst", ".txt"]
     OUTPUT_EXTENSIONS = [".html"]
     ALIASES = ['rst2html']
 
-### @export "rst2latex"
 class Rst2LatexHandler(ProcessStdoutHandler):
     EXECUTABLE = '/usr/bin/env rst2latex.py'
     INPUT_EXTENSIONS = [".rst", ".txt"]
     OUTPUT_EXTENSIONS = [".tex"]
     ALIASES = ['rst2latex']
 
-### @export "rst2beamer"
 class Rst2BeamerHandler(ProcessStdoutHandler):
     EXECUTABLE = '/usr/bin/env rst2beamer'
     INPUT_EXTENSIONS = [".rst", ".txt"]
     OUTPUT_EXTENSIONS = [".tex"]
     ALIASES = ['rst2beamer']
 
-### @export "sloccount"
 class SloccountHandler(ProcessStdoutHandler):
     EXECUTABLE = '/usr/bin/env sloccount'
     VERSION = '/usr/bin/env sloccount --version'
@@ -143,7 +130,6 @@ class SloccountHandler(ProcessStdoutHandler):
     OUTPUT_EXTENSIONS = [".txt"]
     ALIASES = ['sloc', 'sloccount']
 
-### @export "rb"
 class RubyStdoutHandler(ProcessStdoutHandler):
     EXECUTABLE = '/usr/bin/env ruby'
     VERSION = '/usr/bin/env ruby --version'
